/*
  433MHz & RFID Sniffer
  Stefan Thesen, 2016
  Copyright: public domain -> do what you want
  
  For details visit http://blog.thesen.eu 
*/

#include <LiquidCrystal.h>
#include <RCSwitch.h>
#include <SoftwareSerial.h>

// init 433MHz lib
RCSwitch mySwitch = RCSwitch();
unsigned long lOldValue=0;  // to check for consecutive reads on 433MHz

// init SW serial for RFID reader - pin 3 is rx, pin 9 is dummy (we do not need it)
SoftwareSerial RFID = SoftwareSerial(3,9);
String sInput; // for input from RFID module

// init display lib with the numbers of the interface pins
LiquidCrystal lcd(12, 11, 7, 6, 5, 4);

void setup() 
{
  Serial.begin(9600);
  RFID.begin(9600);
  
  // set-up rf receiver
  mySwitch.enableReceive(0);  // 433MHz Receiver on interrupt 0 => that is pin #2
  
  Serial.println("Sniffer for RF 433MHz & RFID - S. Thesen 01/2015");
  
  // set up the LCD's number of columns and rows: 
  lcd.begin(16, 2);
  // Print a message to the LCD.
  lcd.setCursor(0, 0);
  lcd.print("Sniffer");
  lcd.setCursor(0, 1);
  lcd.print("RF 433MHz & RFID");
  lcd.display();
}

void loop() 
{
  char buf[20];
  
  //////////////
  // RF 433 part
  //////////////
  if (mySwitch.available()) 
  {
    int value = mySwitch.getReceivedValue();

    // only react, if at least two times same value received    
    if (value == lOldValue)
    {
      if (value == 0) 
      {
        Serial.print("Unknown encoding");
        lcd.setCursor(0, 1);
        lcd.print("Unknown encoding");          
      } 
      else 
      {
        Serial.print("Received ");
        Serial.print( mySwitch.getReceivedValue() );
        Serial.print(" / ");
        Serial.print( mySwitch.getReceivedBitlength() );
        Serial.print("bit ");
        Serial.print("Protocol: ");
        Serial.println( mySwitch.getReceivedProtocol() );
        
        lcd.clear();
        
        // protocol seems 1 digit, bit max 2 digit
        sprintf(buf,"RF %dBit,Prot %d",mySwitch.getReceivedBitlength(),mySwitch.getReceivedProtocol());
        lcd.setCursor(0, 0);
        lcd.print(buf);
        
        // ulong has max 10 char
        sprintf(buf,"Value=%lu", mySwitch.getReceivedValue()); 
        lcd.setCursor(0, 1);
        lcd.print(buf);
        
        
        // try to calc an elro code
        // wipe out every second bit; these should be zero
        if (mySwitch.getReceivedBitlength()==24)
        {
          int iValue=0;
          unsigned long ulValue = mySwitch.getReceivedValue();
          bool bSuccess=true;
          for (int ii=0;ii<24;ii++)
          {
            unsigned long ulBit = (1l<<ii); 
            if ((ulBit & ulValue)!=0)
            {
              iValue += 1<<(ii/2); 
              
              if ((ii%2)!=0) // there should not be a bit set, if this is Elro compatible
              {
                bSuccess=false;
              }
            }
          }
        
          // can be coded as elro :-)
          if (bSuccess)
          {
            delay(2000);
            lcd.setCursor(0, 0);
            lcd.print("Elro= ");
            Serial.print("Elro coding possible - DIP: ");
            iValue=(~iValue); // invert 
            
            // display in reverse order, i.e. MSB left, LSB right
            // display first 10 bits only - remainder is on/off flagging
            for  (int ii=0;ii<10;ii++)
            {
              if ( ( iValue & (1l<< (11-ii) ) ) !=0 )
              {
                lcd.print("1");
                Serial.print("1");
              }
              else
              {
                lcd.print("0");
                Serial.print("0");
              }
            }
            Serial.println(" ");
          }
        }
        
      }      
    }
    lOldValue = value;
    mySwitch.resetAvailable();  
  }
  
  ////////////
  // RFID part
  //////////// 
  while(RFID.available()>0)
  {
    char cc = RFID.read();
    sInput += cc;
  }
  
  if (sInput.length() > 10) 
  {
    sInput = sInput.substring(1,13);
    Serial.print("Full input from reader = ");
    Serial.println(sInput);
    
    String sConvert;
    sConvert = sInput.substring(4,10);
    long cardcode = strtol(&(sConvert[0]),0,16);      
    sConvert = sInput.substring(2,4);
    long facilitycode = strtol(&(sConvert[0]),0,16);

    Serial.print("FacilityCode = ");
    Serial.print(facilitycode);
    Serial.print(" CardCode = ");    
    Serial.println(cardcode);
    
    lcd.clear();
    lcd.setCursor(0, 0);
    sprintf(buf,"RFID FaC=%lu", facilitycode); 
    lcd.print(buf);
    lcd.setCursor(0, 1);
    sprintf(buf,"CaC=%lu", cardcode); 
    lcd.print(buf);

    sInput = "";
    delay(2000);
  }
}
